/**
  @file App.h
  @author Morgan McGuire, NVIDIA and Williams College, morgan@cs.williams.edu

  This program requires the open source G3D 8.00 library (http://g3d.sf.net)
  to compile.
 */
#ifndef App_h
#define App_h

#include <G3D/G3DAll.h>
#include <GLG3D/GLG3D.h>

class App : public GApp {
    enum Style {NO_STYLE, DEPTH_STYLE, NORMAL_STYLE, GEOMETRIC_STYLE};

    /** Extracted geometry from the model in one mesh */
    Surface::Ref            m_geometry;

    Array<Surface::Ref>     m_model;
    Lighting::Ref           m_lighting;
    ShadowMap::Ref          m_shadowMap;
    GBuffer::Ref            m_gbuffer;
    Style                   m_style;

    float                   m_depthThreshold;
    Shader::Ref             m_depthShader; 

    float                   m_normalThreshold;
    Shader::Ref             m_normalShader; 

    void loadShaders();
    void loadModel();
    void makeGBuffer();
    void makeLighting();
    void makeGui();

public:
    
    App(const GApp::Settings& settings = GApp::Settings());

    virtual void onInit();
    virtual void onPose(Array<Surface::Ref>& posed3D, Array<Surface2D::Ref>& posed2D);
    virtual void onGraphics3D(RenderDevice* rd, Array<Surface::Ref>& surface);
};

#endif
